'use strict';

const chai = require('chai');
const sinonChai = require('sinon-chai');
const proxyquire = require('proxyquire');
const {
    System
} = require('@codefresh-io/authenticated-entity');
const nock = require('nock');

const expect = chai.expect;
chai.use(sinonChai);


describe('Http infra unit test', () => {

            function FakeAuthEntity() {

                this.toJson = () => {
                    return JSON.stringify({
                        _id: 'fake-id'
                    });
                };

                this.encodeToBase64String = () => Buffer.from('somethingToEncode').toString('base64');

                return this;
            }

            function fakeOptionsObject() {
                return {};
            }

            function fakeServiceConfig() {
                return {
                    protocol: 'protocol',
                    uri: 'uri',
                    port: 'port'
                };
            }

            const expectedHeadersEntries = ['X-Authenticated-Entity-Json'];

            function prepareProxy(opt) {
                return proxyquire('./../request', Object.assign({}, opt, {
                    './correlation': {
                        getAuthenticatedEntity() {
                            return new FakeAuthEntity();
                        },
                        getRequestId() {
                            return 'fake-request-id';
                        }
                    }
                }));
            }

            it('Should prepare request', () => {
                function mockPromiseRequest(opt) {
                    expect(opt).to.not.be.empty;
                    //         expect(opt.resolveWithFullResponse).to.be.true;
                    expect(opt.json).to.be.true;
                    expect(opt.headers).to.have.include.keys(expectedHeadersEntries);
                    expect(opt.headers['X-Authenticated-Entity-Json'])
                        .to
                        .be
                        .deep
                        .equal('c29tZXRoaW5nVG9FbmNvZGU=');

                    return Promise.resolve({});
                }

                const requestProxy = prepareProxy({
                    'requestretry': mockPromiseRequest
                });

                return requestProxy({
                    requestOptions: fakeOptionsObject(),
                    serviceConfig: fakeServiceConfig()
                });
            });

            it('Should throw an error on failed request', () => {

                function mockPromiseRequest() {
                    return Promise.reject({
                        error: new Error('Native error should be wrapped'),
                        message: 'Fake massage'
                    });
                }

                const requestProxy = prepareProxy({
                    'requestretry': mockPromiseRequest
                });

                return requestProxy({
                        requestOptions: fakeOptionsObject(),
                        serviceConfig: fakeServiceConfig()
                    })
                    .catch((err) => {
                        expect(err.error).to.be.an.instanceOf(Error);
                    });

            });

            it('Should return the body of the response', () => {
                function mockPromiseRequest() {
                    return Promise.resolve({
                        body: {
                            key: 'val'
                        }
                    });
                }

                const requestProxy = prepareProxy({
                    'requestretry': mockPromiseRequest
                });
                return requestProxy({
                        requestOptions: fakeOptionsObject(),
                        serviceConfig: fakeServiceConfig()
                    })
                    .then((body) => {
                        expect(body).to.not.be.empty;
                    });
            });

            it('should use passed authenticated entity if present', () => {
                const authEntityToOverride = new System('microservice');
                const encodedString = authEntityToOverride.encodeToBase64String();

                function mockPromiseRequest(opt) {
                    expect(opt.headers['X-Authenticated-Entity-Json']).to.equal(encodedString);
                    return Promise.resolve({});
                }

                const requestProxy = prepareProxy({
                    'requestretry': mockPromiseRequest
                });

                return requestProxy({
                    requestOptions: fakeOptionsObject(),
                    serviceConfig: fakeServiceConfig,
                    authenticatedEntity: new System('microservice')
                });
            });

            describe('checking rerty mechanisim', function () {

               this.timeout(20000);
                beforeEach((done) => {
                    process.env['HTTP_RETRY_MODE'] = "";
                    nock('http://notfounderror.com:8080/')
                    .get('/error/')
                    .delayConnection(200)
                    .reply(502)
                    .get('/error/')
                    .reply(502);

                    done();
                });

                it('should retry the request if recevied network error (opt-out)', () => {

                    const maxAttempts = 2;
                    const requestProxy = prepareProxy({});
                    return requestProxy({
                            requestOptions: fakeOptionsObject(),
                            serviceConfig: {
                                protocol: 'http',
                                uri : 'notfounderror.com',
                                port: 8080
                            },
                            maxAttempts: maxAttempts,
                            path: '/error/'
                        })
                        .catch((err) => {
                            expect(err.attempts).to.be.equal(maxAttempts);
                        });

                });

                it('should not retry when set HTTP_SKIP_RETRY', () => {

                    process.env['HTTP_SKIP_RETRY'] = true;
                    const maxAttempts = 5;
                    const requestProxy = prepareProxy({});
                    return requestProxy({
                            requestOptions: fakeOptionsObject(),
                            serviceConfig: {
                                protocol: 'http',
                                uri: 'notfounderror.com',
                                port: 8080

                            },
                            maxAttempts: maxAttempts,
                            path: '/error/'
                        })
                        .catch((err) => {
                            expect(err.attempts).to.be.equal(1);
                        });

                });
            })
        });
